from pydantic import BaseModel, Field
from winion.record_key import RecordKey
from winion.record_value import RecordValue
# from winion.record_headers import RecordHeaders

class Record(BaseModel):
    key: RecordKey = Field()
    value: RecordValue = Field()
    # headers: RecordHeaders = Field()



from typing import List
from winion.record import Record


class ConsumerGroupMixin(object):
    def consume(
        self,
        topic: str,
        records: List[Record],
    ):
        print(f"ProducerMixin#consume topic={topic}")



import asyncio
from pydantic import BaseModel
from typing import Optional


class NullConfig(BaseModel):
    """
    todo...
    """

    pass


class Worker:
    """
    Async worker backing `Producer`, `ConsumerGroup` & `ProducerConsumerGroup`. It provides:
    - config model inititialization
    - main worker loop
    - connection lease acquisition
    - topic storage and consumption
    """

    config: BaseModel

    def __init__(self, config: Optional[BaseModel]):
        if config:
            self.config = config
        else:
            self.config = NullConfig()

    async def start(self):
        while True:
            await asyncio.gather(
                asyncio.sleep(5),
                self.work(),
            )

    async def work(self):
        raise NotImplementedError



from typing import Optional
from winion.record_key import RecordKey
from winion.record_value import RecordValue
from winion.record_headers import RecordHeaders


class ProducerMixin(object):
    def produce(
        self,
        topic: str,
        key: Optional[RecordKey],
        __value__: RecordValue,
        __headers__: RecordHeaders,
    ):
        print(f"ProducerMixin#produce topic={topic}, key={key}")



from winion.consumer_group import ConsumerGroup
from winion.consumer_group_mixin import ConsumerGroupMixin
from winion.producer import Producer
from winion.producer_consumer_group import ProducerConsumerGroup
from winion.producer_mixin import ProducerMixin
from winion.record import Record
from winion.record_headers import RecordHeaders
from winion.record_key import RecordKey
from winion.record_value import RecordValue
from winion.worker import Worker

__all__ = [
    "ConsumerGroup",
    "ConsumerGroupMixin",
    "Producer",
    "ProducerConsumerGroup",
    "ProducerMixin",
    "Record",
    "RecordHeaders",
    "RecordKey",
    "RecordValue",
    "Worker",
]
